@@ -114,6 +114,8 @@ qpwgraph_form::qpwgraph_form (
 
 	m_ins = m_mids = m_outs = 0;
 
+	m_repel_overlapping_nodes = 0;
+
 	m_patchbay_names = new QComboBox(m_ui.patchbayToolbar);
 	m_patchbay_names->setEditable(false);
 	m_patchbay_names->setMinimumWidth(120);
@@ -206,6 +208,9 @@ qpwgraph_form::qpwgraph_form (
 	QObject::connect(m_ui.graphCanvas,
 		SIGNAL(added(qpwgraph_node *)),
 		SLOT(added(qpwgraph_node *)));
+	QObject::connect(m_ui.graphCanvas,
+		SIGNAL(updated(qpwgraph_node *)),
+		SLOT(updated(qpwgraph_node *)));
 	QObject::connect(m_ui.graphCanvas,
 		SIGNAL(removed(qpwgraph_node *)),
 		SLOT(removed(qpwgraph_node *)));
@@ -271,6 +276,10 @@ qpwgraph_form::qpwgraph_form (
 		SIGNAL(toggled(bool)),
 		SLOT(patchbayExclusive(bool)));
 
+	QObject::connect(m_ui.patchbayAutoPinAction,
+		SIGNAL(toggled(bool)),
+		SLOT(patchbayAutoPin(bool)));
+
 	QObject::connect(m_ui.patchbayEditAction,
 		SIGNAL(toggled(bool)),
 		SLOT(patchbayEdit(bool)));
@@ -281,10 +290,6 @@ qpwgraph_form::qpwgraph_form (
 		SIGNAL(triggered(bool)),
 		SLOT(patchbayUnpin()));
 
-	QObject::connect(m_ui.patchbayScanAction,
-		SIGNAL(triggered(bool)),
-		SLOT(patchbayScan()));
-
 	QObject::connect(m_ui.graphQuitAction,
 		SIGNAL(triggered(bool)),
 		SLOT(closeQuit()));
@@ -344,6 +349,10 @@ qpwgraph_form::qpwgraph_form (
 		SIGNAL(triggered(bool)),
 		SLOT(viewZoomRange(bool)));
 
+	QObject::connect(m_ui.viewRepelOverlappingNodesAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewRepelOverlappingNodes(bool)));
+
 	m_ui.viewColorsPipewireAudioAction->setData(qpwgraph_pipewire::audioPortType());
 	m_ui.viewColorsPipewireMidiAction->setData(qpwgraph_pipewire::midiPortType());
 	m_ui.viewColorsPipewireVideoAction->setData(qpwgraph_pipewire::videoPortType());
@@ -526,6 +535,8 @@ void qpwgraph_form::patchbayNew (void)
 	m_patchbay_path.clear();
 	++m_patchbay_untitled;
 
+	m_ui.graphCanvas->patchbayEdit();
+
 	updatePatchbayNames();
 }
 
@@ -648,11 +659,9 @@ void qpwgraph_form::patchbayUnpin (void)
 }
 
 
-void qpwgraph_form::patchbayScan (void)
+void qpwgraph_form::patchbayAutoPin ( bool on )
 {
-	qpwgraph_patchbay *patchbay = m_ui.graphCanvas->patchbay();
-	if (patchbay)
-		patchbay->scan();
+	m_ui.graphCanvas->setPatchbayAutoPin(on);
 
 	stabilize();
 }
@@ -710,6 +719,9 @@ void qpwgraph_form::viewRefresh (void)
 	pipewire_changed();
 	alsamidi_changed();
 
+	if (m_ui.graphCanvas->isRepelOverlappingNodes())
+		++m_repel_overlapping_nodes; // fake nodes added!
+
 	refresh();
 }
 
@@ -784,6 +796,13 @@ void qpwgraph_form::viewSortOrderAction (void)
 }
 
 
+void qpwgraph_form::viewRepelOverlappingNodes ( bool on )
+{
+	m_ui.graphCanvas->setRepelOverlappingNodes(on);
+	if (on) ++m_repel_overlapping_nodes;
+}
+
+
 void qpwgraph_form::helpSystemTray ( bool on )
 {
 #ifdef CONFIG_SYSTEM_TRAY
@@ -913,7 +932,14 @@ void qpwgraph_form::added ( qpwgraph_node *node )
 
 	node->setPos(x, y);
 
-	stabilize();
+	updated(node);
+}
+
+
+void qpwgraph_form::updated ( qpwgraph_node */*node*/ )
+{
+	if (m_ui.graphCanvas->isRepelOverlappingNodes())
+		++m_repel_overlapping_nodes;
 }
 
 
@@ -1026,8 +1052,16 @@ void qpwgraph_form::refresh (void)
 #endif
 
 	if (nchanged > 0) {
-		patchbayScan();
-	//	stabilize();
+		qpwgraph_patchbay *patchbay = m_ui.graphCanvas->patchbay();
+		if (patchbay)
+			patchbay->scan();
+		stabilize();
+	}
+	else
+	if (m_repel_overlapping_nodes > 0) {
+		m_repel_overlapping_nodes = 0;
+		m_ui.graphCanvas->repelOverlappingNodesAll();
+		stabilize();
 	}
 
 	QTimer::singleShot(300, this, SLOT(refresh()));
@@ -1053,7 +1087,6 @@ void qpwgraph_form::stabilize (void)
 	setWindowTitle(title);
 
 	m_ui.patchbayExclusiveAction->setEnabled(is_activated);
-	m_ui.patchbayScanAction->setEnabled(is_activated);
 	m_ui.patchbaySaveAction->setEnabled(is_dirty);
 
 	m_ui.patchbayPinAction->setEnabled(canvas->canPatchbayPin());
@@ -1138,6 +1171,8 @@ bool qpwgraph_form::patchbayOpenFile ( const QString& path, bool clear )
 	m_patchbay_dir = QFileInfo(path).absolutePath();
 	m_patchbay_path = path;
 
+	m_ui.graphCanvas->patchbayEdit();
+
 	patchbay->scan();
 	return true;
 }
@@ -1451,10 +1486,13 @@ void qpwgraph_form::restoreState (void)
 	if (patchbay) {
 		const bool is_activated = m_config->isPatchbayActivated();
 		const bool is_exclusive = m_config->isPatchbayExclusive();
+		const bool is_autopin = m_config->isPatchbayAutoPin();
 		m_ui.patchbayActivatedAction->setChecked(is_activated);
 		m_ui.patchbayExclusiveAction->setChecked(is_exclusive);
+		m_ui.patchbayAutoPinAction->setChecked(is_autopin);
 		patchbay->setActivated(is_activated);
 		patchbay->setExclusive(is_exclusive);
+		m_ui.graphCanvas->setPatchbayAutoPin(is_autopin);
 	}
 
 	m_ui.viewMenubarAction->setChecked(m_config->isMenubar());
@@ -1464,6 +1502,7 @@ void qpwgraph_form::restoreState (void)
 
 	m_ui.viewTextBesideIconsAction->setChecked(m_config->isTextBesideIcons());
 	m_ui.viewZoomRangeAction->setChecked(m_config->isZoomRange());
+	m_ui.viewRepelOverlappingNodesAction->setChecked(m_config->isRepelOverlappingNodes());
 
 	const qpwgraph_port::SortType sort_type
 		= qpwgraph_port::SortType(m_config->sortType());
@@ -1501,6 +1540,7 @@ void qpwgraph_form::restoreState (void)
 
 	viewTextBesideIcons(m_config->isTextBesideIcons());
 	viewZoomRange(m_config->isZoomRange());
+	viewRepelOverlappingNodes(m_config->isRepelOverlappingNodes());
 
 	m_ui.graphCanvas->restoreState();
 
@@ -1526,12 +1566,14 @@ void qpwgraph_form::saveState (void)
 	m_config->setZoomRange(m_ui.viewZoomRangeAction->isChecked());
 	m_config->setSortType(int(qpwgraph_port::sortType()));
 	m_config->setSortOrder(int(qpwgraph_port::sortOrder()));
+	m_config->setRepelOverlappingNodes(m_ui.viewRepelOverlappingNodesAction->isChecked());
 
 	m_config->setStatusbar(m_ui.StatusBar->isVisible());
 	m_config->setToolbar(m_ui.graphToolbar->isVisible());
 	m_config->setPatchbayToolbar(m_ui.patchbayToolbar->isVisible());
 	m_config->setMenubar(m_ui.MenuBar->isVisible());
 
+	m_config->setPatchbayAutoPin(m_ui.patchbayAutoPinAction->isChecked());
 	m_config->setPatchbayExclusive(m_ui.patchbayExclusiveAction->isChecked());
 	m_config->setPatchbayActivated(m_ui.patchbayActivatedAction->isChecked());
 	m_config->setPatchbayPath(m_patchbay_path);
